@@ -11,7 +11,7 @@ from specfile.tags import Comments, Tag, Tags
 
 
 @pytest.mark.parametrize(
-    "tag_name, index",
+    "tag_name, number",
     [
         ("Source", None),
         ("Source0", "0"),
@@ -20,36 +20,90 @@ from specfile.tags import Comments, Tag, Tags
         ("Patch99999", "99999"),
     ],
 )
-def test_tag_source_get_index(tag_name, index):
+def test_tag_source_extract_number(tag_name, number):
     ts = TagSource(Tag(tag_name, "", "", "", Comments()))
-    assert ts._get_index() == index
+    assert ts._extract_number() == number
 
 
 @pytest.mark.parametrize(
-    "ref_name, ref_separator, index, name, separator",
+    "tags, default, result",
+    [
+        (
+            [
+                ("Name", "test"),
+                ("Version", "0.1"),
+                ("Source0", "source0"),
+                ("Source1", "source1"),
+                ("Source2", "source2"),
+            ],
+            True,
+            False,
+        ),
+        (
+            [
+                ("Name", "test"),
+                ("Version", "0.1"),
+                ("Source", "source0"),
+                ("Source", "source1"),
+                ("Source", "source2"),
+            ],
+            False,
+            True,
+        ),
+        (
+            [
+                ("Name", "test"),
+                ("Version", "0.1"),
+                ("Source", "source0"),
+            ],
+            False,
+            False,
+        ),
+        (
+            [
+                ("Name", "test"),
+                ("Version", "0.1"),
+                ("Source", "source0"),
+            ],
+            True,
+            True,
+        ),
+    ],
+)
+def test_sources_detect_implicit_numbering(tags, default, result):
+    sources = Sources(
+        Tags([Tag(t, v, v, ": ", Comments()) for t, v in tags]),
+        [],
+        default_to_implicit_numbering=default,
+    )
+    assert sources._detect_implicit_numbering() == result
+
+
+@pytest.mark.parametrize(
+    "ref_name, ref_separator, number, name, separator",
     [
         ("Source", ": ", 28, "Source28", ":"),
         ("Source0001", ":      ", 2, "Source0002", ":      "),
     ],
 )
-def test_sources_get_tag_format(ref_name, ref_separator, index, name, separator):
-    sources = Sources(None, [])
+def test_sources_get_tag_format(ref_name, ref_separator, number, name, separator):
+    sources = Sources(Tags(), [])
     reference = TagSource(Tag(ref_name, "", "", ref_separator, Comments()))
-    assert sources._get_tag_format(reference, index) == (name, separator)
+    assert sources._get_tag_format(reference, number) == (name, separator)
 
 
 @pytest.mark.parametrize(
-    "tags, index",
+    "tags, number, index",
     [
-        (["Name", "Version"], 2),
-        ([], 0),
+        (["Name", "Version"], 0, 2),
+        ([], 999, 0),
     ],
 )
-def test_sources_get_initial_tag_setup(tags, index):
+def test_sources_get_initial_tag_setup(tags, number, index):
     sources = Sources(
         Tags([Tag(t, "test", "test", ": ", Comments()) for t in tags]), []
     )
-    assert sources._get_initial_tag_setup() == (index, "Source0", ": ")
+    assert sources._get_initial_tag_setup(number) == (index, f"Source{number}", ": ")
 
 
 @pytest.mark.parametrize(
@@ -72,7 +126,7 @@ def test_sources_deduplicate_tag_names(tags, deduplicated_tags):
 
 
 @pytest.mark.parametrize(
-    "tags, sourcelists, index, location, source_index, cls",
+    "tags, sourcelists, index, location, number, cls",
     [
         ([("Name", "test"), ("Version", "0.1")], [], 0, "test", 0, TagSource),
         ([("Name", "test"), ("Version", "0.1")], [[]], 0, "test", 0, ListSource),
@@ -151,7 +205,7 @@ def test_sources_deduplicate_tag_names(tags, deduplicated_tags):
         ),
     ],
 )
-def test_sources_insert(tags, sourcelists, index, location, source_index, cls):
+def test_sources_insert(tags, sourcelists, index, location, number, cls):
     sources = Sources(
         Tags([Tag(t, v, v, ": ", Comments()) for t, v in tags]),
         [
@@ -169,36 +223,121 @@ def test_sources_insert(tags, sourcelists, index, location, source_index, cls):
         if index >= len(sources):
             index = len(sources) - 1
         assert isinstance(sources[index], cls)
-        assert sources[index].index == source_index
+        assert sources[index].number == number
         assert sources[index].location == location
 
 
 @pytest.mark.parametrize(
-    "ref_name, ref_separator, index, name, separator",
+    "tags, number, location, index",
+    [
+        ([("Name", "test"), ("Version", "0.1")], 28, "test", 0),
+        (
+            [
+                ("Name", "test"),
+                ("Version", "0.1"),
+                ("Source0", "source0"),
+                ("Source1", "source1"),
+                ("Source28", "source28"),
+                ("Source999", "source999"),
+            ],
+            0,
+            "test",
+            0,
+        ),
+        (
+            [
+                ("Name", "test"),
+                ("Version", "0.1"),
+                ("Source0", "source0"),
+                ("Source1", "source1"),
+                ("Source28", "source28"),
+                ("Source999", "source999"),
+            ],
+            2,
+            "test",
+            2,
+        ),
+        (
+            [
+                ("Name", "test"),
+                ("Version", "0.1"),
+                ("Source0", "source0"),
+                ("Source1", "source1"),
+                ("Source28", "source28"),
+                ("Source999", "source999"),
+            ],
+            42,
+            "test",
+            3,
+        ),
+        (
+            [
+                ("Name", "test"),
+                ("Version", "0.1"),
+                ("Source0", "source0"),
+                ("Source1", "source1"),
+                ("Source28", "source28"),
+                ("Source999", "source999"),
+            ],
+            1000,
+            "test",
+            4,
+        ),
+        (
+            [
+                ("Name", "test"),
+                ("Version", "0.1"),
+                ("Source28", "source28"),
+                ("Source42", "source42"),
+                ("Source5", "source5"),
+                ("Source", "source"),
+            ],
+            37,
+            "test",
+            1,
+        ),
+    ],
+)
+def test_sources_insert_numbered(tags, number, location, index):
+    sources = Sources(Tags([Tag(t, v, v, ": ", Comments()) for t, v in tags]), [])
+    if location in [v for t, v in tags if t.startswith(Sources.PREFIX)]:
+        with pytest.raises(SpecfileException):
+            sources.insert_numbered(number, location)
+    else:
+        assert sources.insert_numbered(number, location) == index
+        assert isinstance(sources[index], TagSource)
+        assert sources[index].number == number
+        assert sources[index].location == location
+
+
+@pytest.mark.parametrize(
+    "ref_name, ref_separator, number, name, separator",
     [
         ("Patch99", ":      ", 100, "Patch100", ":     "),
         ("Patch9999", ":  ", 28, "Patch0028", ":  "),
         ("Source2", ":     ", 0, "Patch0", ":      "),
     ],
 )
-def test_patches_get_tag_format(ref_name, ref_separator, index, name, separator):
-    patches = Patches(None, [])
+def test_patches_get_tag_format(ref_name, ref_separator, number, name, separator):
+    patches = Patches(Tags(), [])
     reference = TagSource(Tag(ref_name, "", "", ref_separator, Comments()))
-    assert patches._get_tag_format(reference, index) == (name, separator)
+    assert patches._get_tag_format(reference, number) == (name, separator)
 
 
 @pytest.mark.parametrize(
-    "tags, index",
+    "tags, number, index",
     [
-        (["Name", "Version", "Source0"], 3),
-        (["Name", "Version", "Source0", "BuildRequires"], 3),
-        (["Name", "Version"], 2),
-        ([], 0),
+        (["Name", "Version", "Source0"], 0, 3),
+        (["Name", "Version", "Source0", "BuildRequires"], 1, 3),
+        (["Name", "Version"], 2, 2),
+        ([], 999, 0),
     ],
 )
-def test_patches_get_initial_tag_setup(tags, index):
+def test_patches_get_initial_tag_setup(tags, number, index):
     patches = Patches(
         Tags([Tag(t, "test", "test", ": ", Comments()) for t in tags]), []
     )
-    flexmock(patches).should_receive("_get_tag_format").and_return("Patch0", ": ")
-    assert patches._get_initial_tag_setup() == (index, "Patch0", ": ")
+    flexmock(patches).should_receive("_get_tag_format").and_return(
+        f"Patch{number}", ": "
+    )
+    assert patches._get_initial_tag_setup(number) == (index, f"Patch{number}", ": ")
